/****************************************************************************

    PROGRAM:    N2HTML

    FILE:       N2HTML.c

    SYNTAX:     called from the Domino and Notes Workstation Actions menu

    PURPOSE:    Demonstrates how to implement a Domino and Notes menu
                add-in that uses the Midas Rich Text C++ API.

    DESCRIPTION:
        This program provides a procedure to handle two new menu
        items added to the Actions menu in Domino and Notes.  These menu
        actions will be enabled when the user has a view open.  The two
        options allow the user to export the currently selected documents
        to HTML files, and offer a choice of whether or not to
        export just the Body field or the entire document (rendered).

   FUNCTIONS:
       DTMenuProc        - the addin's menu processing procedure.
       ExportField       - Export the Body field to the specified file
       ExportRenderedDoc - Render the whole document with its form, then
                           export to the specified file

****************************************************************************/

/***************************************************************************/
/*                  MIDAS RICH TEXT C++ API                                */
/***************************************************************************/
/* This sample program has been included in the Midas Rich Text C++ API to */
/* demonstrate one way that the Midas Rich Text C++ API can be used.  The  */
/* sample may be altered and used as you wish, but we cannot guarantee that*/
/* the results will work properly.  Use of the Midas Rich Text C++ depends */
/* on the C API and should be used only by those with experience and skill */
/* with the C/C++ programming languages and knowledge of Notes/Domino.     */
/***************************************************************************/


#include <windows.h>            /* required for all Windows apps */

#include <stdio.h>

/* Domino and Notes include files */

#if !defined (LSDEFINE_H)
#include "lsdefine.h"
//#define LSICLI       LSfncED
//#define LSICLIPFN    LSfncEP
class LSXLsiSession;          /* The LSI session object     */
typedef char * LSDYNARRAY;
#endif

#if !defined (RTNOTES_H)
LS_CPP_PASCAL_LINKAGE_BEGIN
#include "rtnotes.h"
#include <addinmen.h>
#include <addin.h>
LS_CPP_PASCAL_LINKAGE_END
#endif

#if !defined (LSXSESS_HPP)
#include "lsxsessi.hpp"
#endif

#if !defined (RTITEM_HPP)
#include "rtitemi.hpp"
#endif

#if !defined (RTCHUNK_HPP)
#include "rtchunki.hpp"
#endif

#define DllImport   __declspec( dllimport )
#define DllExport   __declspec( dllexport )

/* Menu Item IDs */
#define IDM_EXPORTBODYTOHTML   0
#define IDM_EXPORTDOCTOHTML    1

/* create a table that contains the ID and name of the menu items */

typedef struct
{
   WORD ID;
   char *MenuString;
} MENUINFO;

static MENUINFO MenuTable[] =
{
   IDM_EXPORTBODYTOHTML, "Export Body field to HTML",
   IDM_EXPORTDOCTOHTML,  "Export Document to HTML",
};


/* local function prototype */
void ExportField (NAM_COMMAND_INFO *pInfo);
void ExportRenderedDoc (NAM_COMMAND_INFO *pInfo);

#define LINK_DATA_AS_URLLINK           4

STATUS
FixUpLinks(class GeniiRTChunk *SourceChunk_,
           USHORT link_type,
           char *URL_text_,
           USHORT URL_text_len,
           DBID *dbid_,
           UNID *view_unid_,
           UNID *note_unid_,
           char *popup_text,
           USHORT popup_text_len,
           char *server_hint_,
           USHORT server_hint_len,
           char *anchor_text,
           USHORT anchor_text_len)
{
NOTEHANDLE   hNote;
NOTEID       note_id;
STATUS       sError;

  if (link_type == 0 && note_unid_ != NULL)
    {
    SourceChunk_->EventRetShort1 = LINK_DATA_AS_URLLINK;
    sError = NSFNoteOpenByUNID(SourceChunk_->RTItem->hDB, note_unid_, 0, &hNote);
    if (sError == NOERROR)
      {
      NSFNoteGetInfo(hNote, _NOTE_ID, &note_id);
      sprintf(SourceChunk_->S->temp_buffer, "NF%0X.htm", note_id);
      if (anchor_text != NULL && *anchor_text != '\0')
        {
        if (anchor_text_len == 0)
          anchor_text_len = strlen(anchor_text);
        sprintf(SourceChunk_->S->temp_buffer+strlen(SourceChunk_->S->temp_buffer), "#%.*s", anchor_text_len, anchor_text);
        }
      SourceChunk_->EventRetString1 = new char[strlen(SourceChunk_->S->temp_buffer)+1];
      strcpy (SourceChunk_->EventRetString1, SourceChunk_->S->temp_buffer);

      NSFNoteClose(hNote);
      }
    }
  return (sError);
}


NAMRESULT LNCALLBACK DTMenuProc (WORD wMsg, void far *pParam)
{
   static WORD startingID;   /* the Domino and Notes ID for the first
                                addin menu item */

   switch (wMsg)
   {
      case NAMM_INIT:
      {
         NAM_INIT_INFO *pInitInfo;

         /* pParam is a pointer to the NAM_INIT_INFO structure */
         pInitInfo = (NAM_INIT_INFO *)pParam;

         /* If this is the first time through, save wStartingID.
            The menu ID for this item in Windows is wStartingID +
            the menu item ID which is stored in the MenuTable.
            The menu item ID is given to Domino and Notes via the
            wMenuID member of the NAM_INIT_INFO
            strucure */

         if (pInitInfo->wMenuItemNumber == 0)    /* first time through */
            startingID = pInitInfo->wStartingID;

         /* Add each menu item to the Domino and Notes Actions menu */
         /* Tell Domino and Notes about the menu item id and name */

         pInitInfo->wMenuID = MenuTable[pInitInfo->wMenuItemNumber].ID;
         IXPostMessage("Initializing...");
         strcpy (pInitInfo->MenuItemName,
                 MenuTable[pInitInfo->wMenuItemNumber].MenuString);

         /* if there are more menu items to add, request Domino and Notes
        to send NAM_INIT again so that the second menu item may be added */

         if ( pInitInfo->wMenuItemNumber <
             sizeof(MenuTable) / sizeof(MENUINFO) - 1 )
             return (NAM_INIT_CONTINUE);

         return (NAM_INIT_STOP);
      }

      case NAMM_INITMENU:
      {
         NAM_INITMENU_INFO *pInitMenuInfo;
         WORD Flags;

         /* pParam is a pointer to a NAM_INITMENU_INFO structure */
         pInitMenuInfo = (NAM_INITMENU_INFO *)pParam;

        /* enable Export to EML menu item ONLY IF the user is in
           the proper context (ie:  in a view) */

         if (pInitMenuInfo->Data.Context == NAM_IN_VIEW)
            Flags = MF_ENABLED | MF_BYCOMMAND;
         else
            Flags = MF_GRAYED | MF_BYCOMMAND;

         EnableMenuItem( pInitMenuInfo->hMenu,
                         startingID + MenuTable[IDM_EXPORTBODYTOHTML].ID,
                         Flags);

         EnableMenuItem( pInitMenuInfo->hMenu,
                         startingID + MenuTable[IDM_EXPORTDOCTOHTML].ID,
                         Flags);

         return (NAM_NOERROR);

      }


      case NAMM_COMMAND:
      {
         NAM_COMMAND_INFO *pCmdInfo;

         /* pParam is a pointer to a NAM_COMMAND_INFO structure */
         pCmdInfo = (NAM_COMMAND_INFO *)pParam;

         switch (pCmdInfo->wMenuID)
            {
            case IDM_EXPORTBODYTOHTML:
               IXPostMessage("Exporting Body of documents to c:\\temp directory...");
               ExportField(pCmdInfo);
               IXPostMessage("Finished exporting Body of documents to c:\\temp directory.");
               break;

            case IDM_EXPORTDOCTOHTML:
               IXPostMessage("Exporting documents to c:\\temp directory...");
               ExportRenderedDoc(pCmdInfo);
               IXPostMessage("Finished exporting documents to c:\\temp directory.");
               break;

            default:
               break;
         }
         return (NAM_NOERROR);

      } /* case NAMM_COMMAND */

      case NAMM_TERM:
         return (NAM_NOERROR);

      default:                /* default processing */
         return (NAM_NOERROR);
   } /* switch wMsg */
}

void ExportRenderedDoc (NAM_COMMAND_INFO *pInfo)
{
STATUS      sError;
VIEWIXDATA *viewData_;
HANDLE   hSelectedList;  /* handle to ID table of Selected notes */
DBHANDLE hDB;
DWORD    notes_scanned;  /* used in procedure IDScan() */
NOTEID   note_id;        /* gets NOTEIDs stored in ID table */
NOTEHANDLE hNote;
NOTEHANDLE hTempNote;
HANDLE     hCompound;
char       filepath[256];
//USHORT     len;

  viewData_ = &(pInfo->Data.IXData.View);
  if (viewData_->hNoteFile == NULLHANDLE)
    return;

  hSelectedList = viewData_->hSelectedList;
  if (hSelectedList == NULLHANDLE)
    return;

  GeniiSession *gSession;
  GeniiRTItem  *rtitem;

  gSession = new GeniiSession;
  OSGetExecutableDirectory(filepath);
  if (strlen(filepath) > 0)
    {
    if (filepath[strlen(filepath)-1] != '\\')
      strcat(filepath, "\\");
    strcat(filepath, "midisv.lic");
    IXPostMessage(filepath);
    }
  else
    strcpy(filepath, "c:\\lotus\\notes\\data\\midisv.lic");

  if (gSession != NULL)
    gSession->VerifyVersion = MIDAS_VERSION;
  if (gSession == NULL || !gSession->Authenticate(filepath))
    {
    IXPostMessage("Midas NOT authenticated");
    return;
    }

  hDB = viewData_->hNoteFile;
  notes_scanned = 0L;
  while(IDScan(hSelectedList, (FLAG)(notes_scanned++==0L), &note_id))
    {
    /* Open the selected document */
    sError = NSFNoteOpen (hDB, note_id, 0, &hNote);
    if (sError != NOERROR)
      {
      delete (gSession);
      gSession = NULL;
      return;
      }

    /* Create a temporary document, with a compound context */
    sError = NSFNoteCreate(hDB, &hTempNote);
    if (sError != NOERROR)
      {
      NSFNoteClose(hNote);
      delete (gSession);
      gSession = NULL;
      return;
      }

    /* Create the CompoundText context */
    sError = CompoundTextCreate (hTempNote, "Body", &hCompound);
    if (sError != NOERROR)
      {
      NSFNoteClose(hNote);
      NSFNoteClose(hTempNote);
      delete (gSession);
      gSession = NULL;
      return;
      }

    /* Render the note into the context */
    sError = CompoundTextAddRenderedNote(hCompound, hNote, NULLHANDLE, (DWORD) 0);
    if (sError != NOERROR)
      {
     NSFNoteClose(hNote);
      NSFNoteClose(hTempNote);
      CompoundTextDiscard(hCompound);
      delete (gSession);
      gSession = NULL;
      return;
      }

    /* Close the compound context */
    sError = CompoundTextClose (hCompound, NULL, NULL, NULL, 0);
    if (sError != NOERROR)
      {
      NSFNoteClose(hNote);
      NSFNoteClose(hTempNote);
      CompoundTextDiscard(hCompound);
      delete (gSession);
      gSession = NULL;
      return;
      }

    /* Connect to temp field using Midas and Export to Mime */
    rtitem = new GeniiRTItem(gSession);
    sError = rtitem->ConnectBackend(hTempNote, "Body", FALSE);
    rtitem->Everything = rtitem->DefineChunk("Everything");
    rtitem->Everything->ConvertBitmaps("");
    sprintf(filepath, "c:\\temp\\NF%0X.htm", (unsigned long) note_id);
    IXPostMessage(filepath);
    rtitem->ExportToHTML(filepath, "XHTML",
                         "MoveStylesInline='Yes' "
                         "CSSBorders='Yes' "
                         "Generation='Fragment' "
                         "RespectHiddenFormulas='Yes' "
                         "RaiseLinkEvent='Doclink' "
                         "LocalDominoSymbols='Yes' "
                         "AttachmentHandling='To Disk' "
                         "AttachmentDirectory='c:\\temp' "
                         "AttachmentURLDirectory='c:\\temp' "
                         "AttachmentTextInLink='No' "
                         "ImageHandling='TO DISK' "
                         "ImageDirectory='c:\\temp' "
                         "ImageURLDirectory='c:\\temp' ",
                         0,
                         FixUpLinks);

    delete (rtitem);
    rtitem = NULL;
    }

  delete (gSession);
  gSession = NULL;
}


void ExportField (NAM_COMMAND_INFO *pInfo)
{
STATUS      sError;
VIEWIXDATA *viewData_;
HANDLE   hSelectedList;  /* handle to ID table of Selected notes */
DBHANDLE hDB;
DWORD    notes_scanned;  /* used in procedure IDScan() */
NOTEID   note_id;        /* gets NOTEIDs stored in ID table */
NOTEHANDLE hNote;
char       filepath[256];
USHORT     len;

  viewData_ = &(pInfo->Data.IXData.View);
  if (viewData_->hNoteFile == NULLHANDLE)
    return;

  hSelectedList = viewData_->hSelectedList;
  if (hSelectedList == NULLHANDLE)
    return;

  GeniiSession *gSession;
  GeniiRTItem  *rtitem;

  gSession = new GeniiSession;
  len = OSGetDataDirectory(filepath);
  if (len > 0)
    {
    if (filepath[strlen(filepath)-1] != '\\')
      strcat(filepath, "\\");
    strcat(filepath, "midisv.lic");
    IXPostMessage(filepath);
    }
  else
    strcpy(filepath, "c:\\lotus\\notes\\data\\midisv.lic");

  if (gSession != NULL)
    gSession->VerifyVersion = MIDAS_VERSION;
  if (gSession == NULL || !gSession->Authenticate(filepath))
    {
    IXPostMessage("Midas NOT authenticated");
    return;
    }

  hDB = viewData_->hNoteFile;
  notes_scanned = 0L;
  while(IDScan(hSelectedList, (FLAG)(notes_scanned++==0L), &note_id))
    {
    /* Open the selected document */
    sError = NSFNoteOpen (hDB, note_id, 0, &hNote);
    if (sError != NOERROR)
      {
      delete (gSession);
      gSession = NULL;
      return;
      }

    /* Connect to temp field using Midas and Export to Mime */
    rtitem = new GeniiRTItem(gSession);
    sError = rtitem->ConnectBackend(hNote, "Body", FALSE);
    rtitem->Everything = rtitem->DefineChunk("Everything");
    rtitem->Everything->ConvertBitmaps("");
    sprintf(filepath, "c:\\temp\\NF%0X.htm", (unsigned long) note_id);
    IXPostMessage(filepath);
    rtitem->ExportToHTML(filepath, "XHTML",
                         "MoveStylesInline='Yes' "
                         "CSSBorders='Yes' "
                         "Generation='Fragment' "
                         "RespectHiddenFormulas='Yes' "
                         "RaiseLinkEvent='Doclink' "
                         "LocalDominoSymbols='Yes' "
                         "AttachmentHandling='To Disk' "
                         "AttachmentDirectory='c:\\temp' "
                         "AttachmentURLDirectory='c:\\temp' "
                         "AttachmentTextInLink='No' "
                         "ImageHandling='TO DISK' "
                         "ImageDirectory='c:\\temp' "
                         "ImageURLDirectory='c:\\temp' ",
                         0,
                         FixUpLinks);

    delete (rtitem);
    rtitem = NULL;
    NSFNoteClose(hNote);
    }

  delete (gSession);
  gSession = NULL;
}

